@@ -10,6 +10,7 @@
 #' @param boot.reps The number of bootstrap samples to draw.
 #' @param report Should a table of results be printed to the console?
 #' @param prog.bar Show a progress bar of the bootstrap (= TRUE) or not (= FALSE).
+#' @param output.replicates Should the cluster bootstrap coefficient replicates be output (= TRUE) or not (= FALSE)? Only available when impose.null = FALSE.
 #'
 #' @return A list with the elements
 #' \item{p.values}{A matrix of the estimated p-values.}
@@ -19,7 +20,9 @@
 #' @examples
 #' \dontrun{
 #' 
+#' #############################################
 #' # example one: predict cigarette consumption
+#' #############################################
 #' data("CigarettesSW", package = "AER") 
 #' CigarettesSW$rprice <- with(CigarettesSW, price/cpi)
 #' CigarettesSW$rincome <- with(CigarettesSW, income/population/cpi)
@@ -31,8 +34,9 @@
 #' cluster.wd.c <- cluster.wild.ivreg(fm, dat=CigarettesSW, cluster = ~state, report = T)
 #' 
 #' 
-#' 
+#' ################################################
 #' # example two: pooled IV analysis of employment
+#' ################################################
 #' require(plm)
 #' require(AER)
 #' data(EmplUK)
@@ -49,6 +53,7 @@
 #' @importFrom lmtest coeftest
 #' @importFrom sandwich estfun
 #' @importFrom sandwich sandwich
+#' @references Esarey, Justin, and Andrew Menger. 2017. "Practical and Effective Approaches to Dealing with Clustered Data." \emph{Political Science Research and Methods} forthcoming: 1-35. <URL:http://jee3.web.rice.edu/cluster-paper.pdf>.
 #' @references Cameron, A. Colin, Jonah B. Gelbach, and Douglas L. Miller. 2008. "Bootstrap-Based Improvements for Inference with Clustered Errors." \emph{The Review of Economics and Statistics} 90(3): 414-427. <DOI:10.1162/rest.90.3.414>.
 #' @import stats
 #' @importFrom utils write.table
@@ -58,7 +63,12 @@
 #' 
 #
 
-cluster.wild.ivreg<-function(mod, dat, cluster, ci.level = 0.95, impose.null = TRUE, boot.reps = 1000, report = TRUE, prog.bar = TRUE){
+cluster.wild.ivreg<-function(mod, dat, cluster, ci.level = 0.95, impose.null = TRUE, boot.reps = 1000,
+                             report = TRUE, prog.bar = TRUE, output.replicates = FALSE){
+  
+  if(output.replicates == TRUE & impose.null == TRUE){
+    stop("Recovering bootstrap replicates requires setting impose.null = FALSE")
+  }
   
   form <- mod$formula                                            # what is the formula of this model?
   variables <- all.vars(form)                                    # what variables are in this model?
@@ -220,6 +230,10 @@ cluster.wild.ivreg<-function(mod, dat, cluster, ci.level = 0.95, impose.null = T
     boot.dat <- dat                                                                 # copy the data set into a bootstrap resampling dataset
     w.store <- matrix(data=NA, nrow=boot.reps, ncol=length(ind.variables.names))    # store bootstrapped test statistics
     
+    # keep track of the beta bootstrap replicates for possible output
+    rep.store <- matrix(data=NA, nrow=boot.reps, ncol=length(beta.mod))
+    colnames(rep.store) <- ind.variables.names
+    
     resid <- residuals(mod)                                                         # get the residuals for the model
     
     if(prog.bar==TRUE){pb <- txtProgressBar(min = 0, max = boot.reps, initial = 0, style = 3)}
@@ -243,6 +257,8 @@ cluster.wild.ivreg<-function(mod, dat, cluster, ci.level = 0.95, impose.null = T
       beta.boot <- coefficients(boot.mod)                                    # store the bootstrap beta coefficient
       w.store[i,] <- (beta.boot-beta.mod) / se.boot                          # store the bootstrap test statistic
       
+      rep.store[i,] <- beta.boot                                       # store the bootstrap beta for output
+      
     }
     if(prog.bar==TRUE){close(pb)}
     
@@ -291,6 +307,7 @@ cluster.wild.ivreg<-function(mod, dat, cluster, ci.level = 0.95, impose.null = T
   out.list<-list()
   out.list[["p.values"]]<-out
   out.list[["ci"]] <- out.ci
+  if(output.replicates == TRUE){out.list[["replicates"]] <- rep.store}
   return(invisible(out.list))
   
   
